// Models for the simulations of the color pattern on the shells of mollusks
// see also: Meinhardt,H. and Klingler,M. (1987) J. theor. Biol 126, 63-69
// see also: H.Meinhardt: "Algorithmic beauty of sea shells"
// (Springer Verlag) (c) H.Meinhardt, Tübingen
// This is a short version of a program for the simulations of the color
// patterns on tropical sea shells, here 'Oliva porphyria'.
// An autocatalytic activator a(i) leads to a burst-like activation
// that is regulated back by the action of an inhibitor b(i). The life
// time of the inhibitor is regulated via a hormone c, that is
// homogeneously distributed along the growing edge. Whenever the number
// of activated cells cells become too small, active cells remain activated
// until backwards waves are triggered

int imax = 1200; // imax = cells at the growing edge
float[] ax = new float[imax];
float[] bx = new float[imax];
float[] zx = new float[imax];
int KT = 460;  //Number of displays, KT * KP = number of iterations in total
int KP = 12;   //number of iterations between the displays ( = lines on the screen)
int kx = 630;  //number of cells
int dx = 1;    //width of a cell in pixel;   with kp=6 ; kx=315 and dx=2 =>simulation in a smaller field
float DA = .015; //Diffusion of the activator
float RA = .1;   //Decay rate of the inhibitor
float BA = .1;  //Basic production of the activator
float SA = .25;  //Saturation of the autocatalysis
float DB = 0;    //Diffusion of the inhibitor
float RB = .014; //Decay rate of the inhibitor
float SB = .1;   //Michaelis-Menten constant of the inhibition
float RC = .1;   //Decay rate of the hormone

// start:
//----------- Initial condition  --------------------------
for (int i = 1; i<=kx; i++){
 ax[i] = 0;    //Activator, general initial concentration
 bx[i] = .1;   //Inhibitor, general initial concentration
 zx[i] = RA * (.96 + .08 * random(0,1)); //Fluctuation of the autocatalysis
}
float C = .5; //Hormone-concentration, homogeneous in all cells
int ii = 10;
for (int j = 1; j <= 30; j++){  //initially active cells
 ax[ii] = 1;
 ii = ii + int(100 * random(0,1) + 10);
 if (ii > kx) break;
}
float DAC = 1 - RA - 2*DA; // These constant factors are used again and again
float DBC = 1 - RB - 2*DB; // therefore, they are calculated only once
float DBCC = DBC;          // at the begin of the calculation

size(640,480);

//------- boucle -------------------//

//continuo:
background(128);
int x0 = 5; int y1 = 475;       // Initial position of the first line
              
for (int itot = 0; itot <= KT; itot++){ // Number of iteration calculated so far

  for (int iprint = 1; iprint <= KP; iprint++){  // Begin of the iteration
 //Boundary impermeable
 float A1 = ax[1]; // a1 is the concentration  of the actual cell. since this
 float B1 = bx[1]; //   concentration is identical, no diffusion through the border.
 ax[kx + 1] = ax[kx]; //          Concentration in a virtual right cell
 bx[kx + 1] = bx[kx];
 float BSA = 0; //    This will carry the sum of all activations of all cells
 // ---------- Reactions  ------
 for(int i = 1; i <= kx; i++){// i = actual cell, kx = right cell
   float AF = ax[i]; //local activator concentration
   float BF = bx[i]; //local inhibitor concentration
   float AQ = zx[i] * AF * AF / (1 + SA * AF * AF);  // Saturating autocatalysis
   // Calculation of the new activator and inhibitor concentration in cell i:
   ax[i] = AF * DAC + DA * (A1 + ax[i + 1]) + AQ / (SB + BF);
   // 1/BF => Action of the inhibitor; SB = Michaelis-Menten constant
   bx[i] = BF * DBCC + DB * (B1 + bx[i + 1]) + AQ; // new inhibitor conc.
   BSA = BSA + RC * AF; // Hormone production -> Sum of activations
   A1 = AF; // actual concentration will be concentration in left cell
   B1 = BF; // in the concentration change of the next cell
  }
  C = C * (1 - RC) + BSA / kx; // New hormone concentration , 1/kx=normalization
  float RBB = RB / C ;               // on total number of cells
  // RBB => Effective life time of the inhibitor
  DBCC = 1 - 2 * DB - RBB;    // Change in a cell by diffusion
 } // NEXT iprint%            // and decay. Must be recalculated since
                              // lifetime of the inhibitor changes
   // ----------------Plot -------------
   y1 = y1 - 1; //Next plot, one line below
   stroke(255); //Background white
   line(x0, height-y1, x0 + dx * kx, height-y1);
   for(int ix = 1; ix <= kx; ix++){ //Pigment is drawn when a is higher than a threshold
    stroke(228,116,63);
    if( ax[ix] > .5) line(x0 + dx * (ix - 1), height-y1, x0 + dx * ix, height-y1);
   }
}// NEXT itot
//LOCATE 30, 1:  PRINT "c = continue; s = a new start, all other keys = End (c) H.Meinhardt      ";
//DO: a$ = INKEY$: IF a$ > "" THEN EXIT DO
//LOOP
//IF a$ = "c" GOTO continuo
//IF a$ = "s" GOTO start

